/* Copyright (C) 2010 and 2012 Chris Vine

The library comprised in this file or of which this file is part is
distributed by Chris Vine under the GNU Lesser General Public
License as follows:

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License, version 2.1, for more details.

   You should have received a copy of the GNU Lesser General Public
   License, version 2.1, along with this library (see the file LGPL.TXT
   which came with this source code package in the src/utils sub-directory);
   if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

However, it is not intended that the object code of a program whose
source code instantiates a template from this file or uses macros or
inline functions (of any length) should by reason only of that
instantiation or use be subject to the restrictions of use in the GNU
Lesser General Public License.  With that in mind, the words "and
macros, inline functions and instantiations of templates (of any
length)" shall be treated as substituted for the words "and small
macros and small inline functions (ten lines or less in length)" in
the fourth paragraph of section 5 of that licence.  This does not
affect any other reason why object code may be subject to the
restrictions in that licence (nor for the avoidance of doubt does it
affect the application of section 2 of that licence to modifications
of the source code in this file).

*/

#ifndef CGU_FUTURE_TPP
#define CGU_FUTURE_TPP

namespace Cgu {

namespace Thread {

extern "C" {
inline void cgu_future_cancel_cleanup_handler(void* arg) {
  Cgu::Callback::Callback* cb = static_cast<Cgu::Callback::Callback*>(arg);
  cb->dispatch();
  delete cb;
}
}

template <class Val>
void Future<Val>::cancel_cleanup() {
  {
    Mutex::Lock lock(mutex);
    error = true;
    done = true;
    running = false;
    cond.broadcast();  // any threads waiting on Future<Val>::get() will
                       // have their own reference, so we can unref() now
  }
  unref();
}

template <class Val>
template <class Ret, class T>
void Future<Val>::run_wrapper(const arg_pair<T&, Ret (T::*)()>& args, 
			      Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = ((args.obj).*(args.func))();
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class Arg1, class T>
void Future<Val>::run_wrapper(const arg_triple<T&, Ret (T::*)(Arg1), Arg1>& args,
			      Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = ((args.obj).*(args.func))(args.arg1);
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class Arg1, class Arg2, class T>
void Future<Val>::run_wrapper(const arg_quadruple<T&, Ret (T::*)(Arg1, Arg2), Arg1, Arg2>& args,
			      Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = ((args.obj).*(args.func))(args.arg1, args.arg2);
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class T>
void Future<Val>::run_wrapper_const(const arg_pair<const T&, Ret (T::*)() const>& args,
				    Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = ((args.obj).*(args.func))();
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class Arg1, class T>
void Future<Val>::run_wrapper_const(const arg_triple<const T&, Ret (T::*)(Arg1) const, Arg1>& args,
				    Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = ((args.obj).*(args.func))(args.arg1);
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class Arg1, class Arg2, class T>
void Future<Val>::run_wrapper_const(const arg_quadruple<const T&, Ret (T::*)(Arg1, Arg2) const, Arg1, Arg2>& args,
				    Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = ((args.obj).*(args.func))(args.arg1, args.arg2);
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret>
void Future<Val>::run_wrapper_static(Ret (*func)(),
				     Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = func();
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class Arg1>
void Future<Val>::run_wrapper_static(const arg_triple<Empty, Ret (*)(Arg1), Arg1>& args,
				     Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = args.func(args.arg1);
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

template <class Val>
template <class Ret, class Arg1, class Arg2>
void Future<Val>::run_wrapper_static(const arg_quadruple<Empty, Ret (*)(Arg1, Arg2), Arg1, Arg2>& args,
				     Future<Val>* instance) {
  Cgu::Callback::Callback* cb;
  {
    CancelBlock b;
    try {
      cb = Callback::make(*instance, &Future<Val>::cancel_cleanup);
    }
    catch (...) {
      instance->cancel_cleanup();      
      return;
    }
  }

  pthread_cleanup_push(cgu_future_cancel_cleanup_handler, cb);

  try {
    // we don't want to hold the mutex when executing the callback (we may
    // be creating deadlocks from out of order locking) and we don't need
    // to when assigning to val, because before the get() method can proceed
    // this thread will have released the mutex below and the thread calling
    // get() will have since acquired it
    instance->val = args.func(args.arg1, args.arg2);
  }
  catch (Exit&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  catch (std::exception&) {
    Mutex::Lock lock(instance->mutex);
    instance->error = true;
  }
  // we won't catch with the elypsis argument because NPTL forced stack
  // unwinding would require the exception to be rethrown.  For any exceptions
  // other than Thread::Exit and those derived from std::exception we will
  // have to rely on the user stopping them reaching this far.

  // don't allow any further thread cancellation, and particularly when
  // any callback connected to done_emitter is executed
  CancelBlock b;
  pthread_cleanup_pop(false);
  delete cb;

  instance->mutex.lock();
  instance->done = true;
  instance->cond.broadcast();
  if (!instance->error) {
    instance->mutex.unlock();
    try {
      instance->done_emitter();
      Mutex::Lock lock(instance->mutex);
      // we use running also to indicate whether done_emitter() has
      // completed
      instance->running = false;
    }
    catch (...) {
      Mutex::Lock lock(instance->mutex);
      instance->emitter_error = true;
      instance->running = false;
    }
  }
  else {
    instance->running = false;
    instance->mutex.unlock();
  }
  instance->unref();  // any threads waiting on Future<Val>::get() will
                      // have their own reference, so we can unref() now
}

  
template <class Val>
template <class Ret, class T>
IntrusivePtr<Future<Val> > Future<Val>::make(T& t,
					     Ret (T::*func)()) {
  
  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_pair<T&, Ret (T::*)()>&,
							      Future<Val>*)>(&run_wrapper<Ret, T>),
					  arg_pair<T&, Ret (T::*)()>(t, func),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class Arg1, class T>
IntrusivePtr<Future<Val> > Future<Val>::make(T& t,
					     Ret (T::*func)(Arg1),
					     typename Cgu::Param<Arg1>::ParamType arg1) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_triple<T&, Ret (T::*)(Arg1), Arg1>&,
							      Future<Val>*)>(&run_wrapper<Ret, Arg1, T>),
					  arg_triple<T&, Ret (T::*)(Arg1), Arg1>(t, func, arg1),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class Arg1, class Arg2, class T>
IntrusivePtr<Future<Val> > Future<Val>::make(T& t,
					     Ret (T::*func)(Arg1, Arg2),
					     typename Cgu::Param<Arg1>::ParamType arg1,
					     typename Cgu::Param<Arg2>::ParamType arg2) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_quadruple<T&, Ret (T::*)(Arg1, Arg2), Arg1, Arg2>&,
							      Future<Val>*)>(&run_wrapper<Ret, Arg1, Arg2, T>),
					  arg_quadruple<T&, Ret (T::*)(Arg1, Arg2), Arg1, Arg2>(t, func, arg1, arg2),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class T>
IntrusivePtr<Future<Val> > Future<Val>::make(const T& t,
					     Ret (T::*func)() const) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_pair<const T&, Ret (T::*)() const>&,
							      Future<Val>*)>(&run_wrapper_const<Ret, T>),
					  arg_pair<const T&, Ret (T::*)() const>(t, func),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class Arg1, class T>
IntrusivePtr<Future<Val> > Future<Val>::make(const T& t,
					     Ret (T::*func)(Arg1) const,
					     typename Cgu::Param<Arg1>::ParamType arg1) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_triple<const T&, Ret (T::*)(Arg1) const, Arg1>&,
							      Future<Val>*)>(&run_wrapper_const<Ret, Arg1, T>),
					  arg_triple<const T&, Ret (T::*)(Arg1) const, Arg1>(t, func, arg1),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class Arg1, class Arg2, class T>
IntrusivePtr<Future<Val> > Future<Val>::make(const T& t,
					     Ret (T::*func)(Arg1, Arg2) const,
					     typename Cgu::Param<Arg1>::ParamType arg1,
					     typename Cgu::Param<Arg2>::ParamType arg2) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_quadruple<const T&, Ret (T::*)(Arg1, Arg2) const, Arg1, Arg2>&,
							      Future<Val>*)>(&run_wrapper_const<Ret, Arg1, Arg2, T>),
					  arg_quadruple<const T&, Ret (T::*)(Arg1, Arg1) const, Arg1, Arg2>(t, func, arg1, arg2),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret>
IntrusivePtr<Future<Val> > Future<Val>::make(Ret (*func)()) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make(static_cast<void(*)(Ret (*)(),
							  Future<Val>*)>(&run_wrapper_static<Ret>),
				      func,
				      instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class Arg1>
IntrusivePtr<Future<Val> > Future<Val>::make(Ret (*func)(Arg1),
					     typename Cgu::Param<Arg1>::ParamType arg1) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_triple<Empty, Ret (*)(Arg1), Arg1>&,
							      Future<Val>*)>(&run_wrapper_static<Ret, Arg1>),
					  arg_triple<Empty, Ret (*)(Arg1), Arg1>(Empty(), func, arg1),
					  instance.get()));
  return instance;
}

template <class Val>
template <class Ret, class Arg1, class Arg2>
IntrusivePtr<Future<Val> > Future<Val>::make(Ret (*func)(Arg1, Arg2),
					     typename Cgu::Param<Arg1>::ParamType arg1,
					     typename Cgu::Param<Arg2>::ParamType arg2) {

  IntrusivePtr<Future<Val> > instance(new Future<Val>);
  instance->cb_a.reset(Callback::make_ref(static_cast<void(*)(const arg_quadruple<Empty, Ret (*)(Arg1, Arg2), Arg1, Arg2>&,
							      Future<Val>*)>(&run_wrapper_static<Ret, Arg1, Arg2>),
					  arg_quadruple<Empty, Ret (*)(Arg1, Arg2), Arg1, Arg2>(Empty(), func, arg1, arg2),
					  instance.get()));
  return instance;
}


template <class Val>
bool Future<Val>::run() {

  // take a reference so that this object will exist at least
  // until the worker thread has completed (we release in one
  // of the run_wrapper() methods, in cancel_cleanup() or on
  // throwing below)
  ref();

  Mutex::Lock lock(mutex);
  // we don't want to throw an exception if run() has already been
  // called - that may not be a programming error - the user may not
  // care whether it has been called, just that it should have been
  // called (Future<>::get() can be called any number of times by
  // any number of threads).  So provide a return value which can be
  // inspected where it is relevant to the particular usage case
  if (running || done || error) { // if run() has previously thrown, although 'running'
                                  // and 'done' will be false, error will be true
    unref();
    return false;
  }
  running = true;

  try {
    thread_a = Cgu::Thread::Thread::start(cb_a.release(), false);
  }
  catch (...) {
    error = true;
    running = false;
    unref();
    throw;
  }
  if (!thread_a.get()) {
    error = true;
    running = false;
    unref();
    throw FutureThreadError();
  }
  return true;
}

template <class Val>
Val Future<Val>::get() {

  // make sure run() has been called: if run() has previously thrown,
  // although 'running' and 'done' will be false, 'error' will be true
  mutex.lock();
  if (!running && !done && !error) {
    CancelBlock b;
    mutex.unlock();
    run();                 // there is no race - if another thread beats us
                           // to it after unlocking, run() will just return
    mutex.lock();
  }
  Mutex::Lock lock(mutex, locked);
  if (!error) {            // we don't want to hang if run() threw because
                           // it could not start a worker thread
    while (!done) cond.wait(mutex);
  }
  CancelBlock b;
  return val;
}

template <class Val>
bool Future<Val>::cancel() {
  Mutex::Lock lock(mutex);
  if (!done && thread_a.get()) {
    thread_a->cancel();
    return true;
  }
  return false;     
}

template <class Val>
void Future<Val>::execute_done(const Cgu::Callback::SafeFunctorArg<const Val&>& f) {
  // take an additional reference in an intrusive pointer in case f()
  // throws, so we can call unref() before the call to f()
  Cgu::IntrusivePtr<Future<Val> > temp(this);
  unref();  // remove reference added in post_done()

  // we can access 'val' directly here, rather than calling 'get()'
  // because attaching to and dispatching via the main loop will
  // guarantee memory visibility
  f(val);
}

template <class Val>
void Future<Val>::post_done(const Cgu::Callback::SafeFunctorArg<const Val&>& f,
			    const when_mainloop_args& l) {
  // take an additional reference in an intrusive pointer in case
  // Callback::post() throws.  We must acquire a reference before
  // Callback::post() returns to avoid a race condition with the main
  // loop thread.
  Cgu::IntrusivePtr<Future<Val> > temp(this);
  Cgu::Callback::post(Cgu::Callback::make_ref(*this, &Future<Val>::execute_done, f),
		      l.priority,
		      l.context);
  // the Future object is guaranteed to be in existence until after
  // done_emitter has completed all its emissions, but we need to add
  // another reference in this method to also make sure that the
  // Future object lasts until execute_done executes.  We don't need
  // to put the call to Callback::post in a try block, because the
  // run_wrapper methods catch any exceptions when done_emitter emits
  // and set emitter_error in such a case.
  ref();
}

template <class Val>
Cgu::Callback::SafeFunctor Future<Val>::when(const Cgu::Callback::CallbackArg<const Val&>* cb,
					     gint priority,
					     GMainContext* context) {

  Cgu::Callback::SafeFunctorArg<const Val&> f(cb);   // take ownership

  Mutex::Lock lock(mutex);
  if (done) throw FutureWhenError();
  return done_emitter.connect(Cgu::Callback::make_ref(*this,
						      &Future<Val>::post_done, 
						      f,
						      when_mainloop_args(priority, context)));
}

template <class Val>
void Future<Val>::execute_done_rel(const Cgu::SharedLockPtr<Cgu::SafeEmitterArg<const Val&> >& s) {
  // take an additional reference in an intrusive pointer in case
  // emit() throws, so we can call unref() before the call to emit()
  Cgu::IntrusivePtr<Future<Val> > temp(this);
  unref();  // remove reference added in post_done_rel()

  // we can access 'val' directly here, rather than calling 'get()'
  // because attaching to and dispatching via the main loop will
  // guarantee memory visibility
  s->emit(val);
}

template <class Val>
void Future<Val>::post_done_rel(const Cgu::SharedLockPtr<Cgu::SafeEmitterArg<const Val&> >& s,
				const when_mainloop_args& l) {

  // take an additional reference in an intrusive pointer in case
  // Callback::post() throws.  We must acquire a reference before
  // Callback::post() returns to avoid a race condition with the main
  // loop thread
  Cgu::IntrusivePtr<Future<Val> > temp(this);
  Cgu::Callback::post(Cgu::Callback::make_ref(*this, &Future<Val>::execute_done_rel, s),
		      l.priority,
		      l.context);
  // the Future object is guaranteed to be in existence until after
  // done_emitter has completed all its emissions, but we need to add
  // another reference in this method to also make sure that the
  // Future object lasts until execute_done_rel() executes.  We don't
  // need to put the call to Callback::post in a try block, because
  // the run_wrapper methods catch any exceptions when done_emitter
  // emits and set emitter_error in such a case.
  ref();
}

template <class Val>
Cgu::Callback::SafeFunctor Future<Val>::when(const Cgu::Callback::CallbackArg<const Val&>* cb,
					     Cgu::Releaser& r,
					     gint priority,
					     GMainContext* context) {

  Cgu::Callback::SafeFunctorArg<const Val&> f(cb);   // take ownership
  Cgu::SharedLockPtr<Cgu::SafeEmitterArg<const Val&> > s(new Cgu::SafeEmitterArg<const Val&>);
  s->connect(f, r);

  Mutex::Lock lock(mutex);
  if (done) throw FutureWhenError();
  return done_emitter.connect(Cgu::Callback::make_ref(*this,
						      &Future<Val>::post_done_rel,
						      s,
						      when_mainloop_args(priority, context)));
}

template <class Val>
void Future<Val>::fail_cb(const Cgu::IntrusivePtr<Future<Val> >& future,
			  const Cgu::Callback::SafeFunctor& func,
			  bool& ret) {
  Mutex::TrackLock lock(future->mutex);
  if (!future->running && future->done) { // everything finished
    // remove timeout
    ret = false;
    if (future->error || future->emitter_error) {
      // we failed, execute 'func'
      lock.unlock();
      func();
    }
  }
  else if (!future->running && !future->done && future->error) {
    // if (!running && !done && error) then run() has thrown and the
    // thread never launched - stop timeout
    ret = false;
  }
}

template <class Val>
void Future<Val>::fail(const Cgu::Callback::Callback* cb,
		       Cgu::Releaser& r,
		       GMainContext* context) {

  Cgu::Callback::SafeFunctor f(cb);   // take ownership
  Cgu::start_timeout(100,
		     Cgu::Callback::make_ref(&Future<Val>::fail_cb,
					     Cgu::IntrusivePtr<Future<Val> >(this),
					     f),
		     r,
		     G_PRIORITY_DEFAULT,
		     context);
}

template <class Val>
void Future<Val>::fail(const Cgu::Callback::Callback* cb,
		       GMainContext* context) {

  Cgu::Callback::SafeFunctor f(cb);   // take ownership
  Cgu::start_timeout(100,
		     Cgu::Callback::make_ref(&Future<Val>::fail_cb,
					     Cgu::IntrusivePtr<Future<Val> >(this),
					     f),
		     G_PRIORITY_DEFAULT,
		     context);
}

template <class Val>
bool Future<Val>::is_done() const {
  Mutex::Lock lock(mutex);
  return done;
}

template <class Val>
bool Future<Val>::is_emitter_done() const {
  Mutex::Lock lock(mutex);
  return (done && !running);   //'running' does not return to a false value until
                               // 'done_emitter' has completed emitting
}

template <class Val>
bool Future<Val>::is_error() const {
  Mutex::Lock lock(mutex);
  return error;
}

template <class Val>
bool Future<Val>::is_emitter_error() const {
  Mutex::Lock lock(mutex);
  return emitter_error;
}

} // namespace Thread

} // namespace Cgu

#endif // CGU_FUTURE_TPP
